import { Component, ChangeDetectionStrategy, ComponentFactoryResolver, Input, ViewChild, ViewContainerRef, HostBinding, HostListener, Injectable, ApplicationRef, ɵɵdefineInjectable, ɵɵinject, NgModule } from '@angular/core';
import { trigger, state, style, transition, animate } from '@angular/animations';
import { NavigationStart, Router, RouterModule } from '@angular/router';
import { filter } from 'rxjs/operators';
import { CommonModule } from '@angular/common';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ModalComponent {
    /**
     * @param {?} resolver
     * @param {?} router
     */
    constructor(resolver, router) {
        this.resolver = resolver;
        this.router = router;
        this.scaling = false;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (!!this.width) {
            this.hostWidth = this.width;
        }
        if (!!this.height) {
            this.hostHeight = this.height;
        }
        /** @type {?} */
        const factory = this.resolver.resolveComponentFactory(this.injectComponent.component);
        this.componentRef = this.viewContainerRef.createComponent(factory);
        if (!!this.injectComponent.inputs) {
            Object.keys(this.injectComponent.inputs)
                .forEach((/**
             * @param {?} key
             * @return {?}
             */
            key => this.componentRef.instance[key] = this.injectComponent.inputs[key]));
        }
        this.componentRef.instance.modal = this;
        this.scaling = true;
        this.subscription = this.router.events
            .pipe(filter((/**
         * @param {?} p
         * @return {?}
         */
        p => p instanceof NavigationStart)))
            .subscribe((/**
         * @return {?}
         */
        () => {
            this.componentRef.destroy();
            this.injectComponent.reject();
        }));
        if (this.showOverlay) {
            this.showOverlay();
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this.hideOverlay) {
            this.hideOverlay();
        }
        this.subscription.unsubscribe();
    }
    /**
     * @template T
     * @param {?} result
     * @return {?}
     */
    close(result) {
        this.result = result;
        this.scaling = false;
    }
    /**
     * @param {?} e
     * @return {?}
     */
    animationDone(e) {
        if (!e.toState) {
            this.injectComponent.resolve(this.result);
            this.componentRef.destroy();
        }
    }
}
ModalComponent.decorators = [
    { type: Component, args: [{
                // tslint:disable-next-line:component-selector
                selector: 'a4-modal',
                template: `<ng-template #container></ng-template>`,
                animations: [
                    trigger('scaling', [
                        state('0', style({ transform: 'translate(-50%, -50%) scale(0)' })),
                        state('1', style({ transform: 'translate(-50%, -50%) scale(1)' })),
                        transition(':enter', [style({ transform: 'translate(-50%, -50%) scale(0)' }), animate('250ms ease')]),
                        transition('1 => 0', animate('250ms ease'))
                    ])
                ],
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [`
  :host {
    display: block;
    position: fixed;
    z-index: 1000;
    background-color: white;
    left: 50vw;
    top: 50vh;
    border: 1px solid rgba(0,0,0,0.1);
    border-radius: 4px;
    box-shadow: 0 5px 5px -3px rgba(0,0,0,.2), 0 8px 10px 1px rgba(0,0,0,.14), 0 3px 14px 2px rgba(0,0,0,.12);
  }
  `]
            }] }
];
/** @nocollapse */
ModalComponent.ctorParameters = () => [
    { type: ComponentFactoryResolver },
    { type: Router }
];
ModalComponent.propDecorators = {
    injectComponent: [{ type: Input }],
    width: [{ type: Input }],
    height: [{ type: Input }],
    showOverlay: [{ type: Input }],
    hideOverlay: [{ type: Input }],
    viewContainerRef: [{ type: ViewChild, args: ['container', { read: ViewContainerRef, static: true },] }],
    scaling: [{ type: HostBinding, args: ['@scaling',] }],
    hostWidth: [{ type: HostBinding, args: ['style.width',] }],
    hostHeight: [{ type: HostBinding, args: ['style.height',] }],
    animationDone: [{ type: HostListener, args: ['@scaling.done', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ModalService {
    /**
     * @param {?} resolver
     * @param {?} applicationRef
     */
    constructor(resolver, applicationRef) {
        this.resolver = resolver;
        this.applicationRef = applicationRef;
    }
    /**
     * @private
     * @return {?}
     */
    get viewContainerRef() {
        /** @type {?} */
        const appInstance = this.applicationRef.components[0].instance;
        if (!appInstance.viewContainerRef) {
            /** @type {?} */
            const appName = this.applicationRef.componentTypes[0].name;
            throw new Error(`Missing 'viewContainerRef' declaration in ${appName} constructor`);
        }
        return appInstance.viewContainerRef;
    }
    /**
     * @template T
     * @param {?} component
     * @param {?=} inputs
     * @param {?=} width
     * @param {?=} height
     * @return {?}
     */
    open(component, inputs, width, height) {
        /** @type {?} */
        let componentRef;
        return new Promise((/**
         * @param {?} mainResolve
         * @param {?} mainReject
         * @return {?}
         */
        (mainResolve, mainReject) => {
            new Promise((/**
             * @param {?} resolve
             * @param {?} reject
             * @return {?}
             */
            (resolve, reject) => {
                setTimeout((/**
                 * @return {?}
                 */
                () => {
                    /** @type {?} */
                    const factory = this.resolver.resolveComponentFactory(ModalComponent);
                    componentRef = this.viewContainerRef.createComponent(factory);
                    componentRef.instance['injectComponent'] = {
                        component, inputs, resolve, reject
                    };
                    componentRef.instance.width = width;
                    componentRef.instance.height = height;
                    componentRef.instance.showOverlay = this.showOverlay;
                    componentRef.instance.hideOverlay = this.hideOverlay;
                }), 0);
            })).then((/**
             * @param {?} result
             * @return {?}
             */
            (result) => {
                componentRef.destroy();
                mainResolve(result);
            })).catch((/**
             * @param {?} reason
             * @return {?}
             */
            reason => {
                componentRef.destroy();
                mainReject(reason);
            }));
        }));
    }
    /**
     * @param {?} showOverlay
     * @param {?} hideOverlay
     * @return {?}
     */
    setOverlay(showOverlay, hideOverlay) {
        this.showOverlay = showOverlay;
        this.hideOverlay = hideOverlay;
    }
}
ModalService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
ModalService.ctorParameters = () => [
    { type: ComponentFactoryResolver },
    { type: ApplicationRef }
];
/** @nocollapse */ ModalService.ngInjectableDef = ɵɵdefineInjectable({ factory: function ModalService_Factory() { return new ModalService(ɵɵinject(ComponentFactoryResolver), ɵɵinject(ApplicationRef)); }, token: ModalService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ModalModule {
}
ModalModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    RouterModule,
                    BrowserAnimationsModule
                ],
                declarations: [
                    ModalComponent
                ],
                entryComponents: [
                    ModalComponent
                ],
                providers: [
                    ModalService
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ModalComponent, ModalModule, ModalService };
//# sourceMappingURL=a4-modal.js.map
